/**
 * Copyright (c) 2014 - 2017, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

/** @file
 * @defgroup pin_change_int_example_main main.c
 * @{
 * @ingroup pin_change_int_example
 * @brief Pin Change Interrupt Example Application main file.
 *
 * This file contains the source code for a sample application using interrupts triggered by GPIO pins.
 *
 */

#include <stdbool.h>
#include "nrf.h"
#include "nrf_delay.h"
#include "nrf_drv_clock.h"
#include "nrf_drv_gpiote.h"
#include "nrf_drv_ppi.h"
#include "nrf_drv_timer.h"
#include "app_error.h"
#include "boards.h"
#include "app_trace.h"

#define PIN_IN_1  28
#define PIN_IN_2  29
#define PIN_OUT_1 30
#define PIN_OUT_2 12

static const nrf_drv_timer_t m_timer = NRF_DRV_TIMER_INSTANCE(0);

nrf_ppi_channel_t ppi_channel_1;
nrf_ppi_channel_t ppi_channel_2;
nrf_ppi_channel_t ppi_channel_3;

void in_pin_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
{

    if(pin  == PIN_IN_2)
    {
        app_trace_log("Timer value: %d\r\n",nrf_drv_timer_capture_get(&m_timer, NRF_TIMER_CC_CHANNEL0)); 
        nrf_drv_timer_clear(&m_timer);
    }
}

void timer_handler(nrf_timer_event_t event_type, void * p_context)
{

}

/**
 * @brief Function for configuring: PIN_IN pin for input, PIN_OUT pin for output,
 * and configures GPIOTE to give an interrupt on pin change.
 */
static void gpiote_init(void)
{
    ret_code_t err_code;

    err_code = nrf_drv_gpiote_init();
    APP_ERROR_CHECK(err_code);

    nrf_drv_gpiote_out_config_t out_config = GPIOTE_CONFIG_OUT_SIMPLE(false);

    err_code = nrf_drv_gpiote_out_init(PIN_OUT_1, &out_config);
    APP_ERROR_CHECK(err_code);
    
    err_code = nrf_drv_gpiote_out_init(PIN_OUT_2, &out_config);
    APP_ERROR_CHECK(err_code);
    
    nrf_drv_gpiote_out_clear(PIN_OUT_1);
    nrf_drv_gpiote_out_clear(PIN_OUT_2);

    nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_SENSE_LOTOHI(true);
    in_config.pull = NRF_GPIO_PIN_PULLDOWN;

    err_code = nrf_drv_gpiote_in_init(PIN_IN_1, &in_config, in_pin_handler);
    APP_ERROR_CHECK(err_code);
    err_code = nrf_drv_gpiote_in_init(PIN_IN_2, &in_config, in_pin_handler);
    APP_ERROR_CHECK(err_code);
    
    nrf_drv_gpiote_in_event_enable(PIN_IN_1, false);
    nrf_drv_gpiote_in_event_enable(PIN_IN_2, true);
}

void timer_init()
{
    ret_code_t err_code;

    //nrf_drv_timer_config_t timer_cfg = NRF_DRV_TIMER_DEFAULT_CONFIG;
    //timer_cfg.bit_width = NRF_TIMER_BIT_WIDTH_32;
    err_code = nrf_drv_timer_init(&m_timer, NULL, timer_handler);
    APP_ERROR_CHECK(err_code);
}

void ppi_init()
{
    ret_code_t err_code;

    err_code = nrf_drv_ppi_init();
    APP_ERROR_CHECK(err_code);
    
    err_code = nrf_drv_ppi_channel_alloc(&ppi_channel_1);
    APP_ERROR_CHECK(err_code);
    err_code = nrf_drv_ppi_channel_alloc(&ppi_channel_2);
    APP_ERROR_CHECK(err_code);
    err_code = nrf_drv_ppi_channel_alloc(&ppi_channel_3);
    APP_ERROR_CHECK(err_code);
    
    uint32_t gpiote_evt_addr_1          = nrf_drv_gpiote_in_event_addr_get(PIN_IN_1);
    uint32_t gpiote_evt_addr_2          = nrf_drv_gpiote_in_event_addr_get(PIN_IN_2);
    
    uint32_t timer_start_task_addr      = nrf_drv_timer_task_address_get(&m_timer, NRF_TIMER_TASK_START);
    uint32_t timer_stop_task_addr       = nrf_drv_timer_task_address_get(&m_timer, NRF_TIMER_TASK_STOP);
    uint32_t timer_compare_task_addr    = nrf_drv_timer_task_address_get(&m_timer, NRF_TIMER_TASK_CAPTURE0);
    
    err_code = nrf_drv_ppi_channel_assign(ppi_channel_1, gpiote_evt_addr_1, timer_start_task_addr);
    APP_ERROR_CHECK(err_code);
    err_code = nrf_drv_ppi_channel_assign(ppi_channel_2, gpiote_evt_addr_2, timer_compare_task_addr);
    APP_ERROR_CHECK(err_code);
    err_code = nrf_drv_ppi_channel_assign(ppi_channel_3, gpiote_evt_addr_2, timer_stop_task_addr);
    APP_ERROR_CHECK(err_code);
    
    err_code = nrf_drv_ppi_channel_enable(ppi_channel_1);
    APP_ERROR_CHECK(err_code);
    err_code = nrf_drv_ppi_channel_enable(ppi_channel_2);
    APP_ERROR_CHECK(err_code);
    
    err_code = nrf_drv_ppi_channel_enable(ppi_channel_3);
    APP_ERROR_CHECK(err_code);
}

//static void hfclk_config(void)
//{
//    ret_code_t err_code = nrf_drv_clock_init();
//    APP_ERROR_CHECK(err_code);

//    nrf_drv_clock_hfclk_request(NULL);
//}

/**
 * @brief Function for application main entry.
 */
int main(void)
{
    app_trace_init();
    
//    hfclk_config();
    
    //while(!nrf_drv_clock_hfclk_is_running());
    
    gpiote_init();
    timer_init();
    ppi_init();
    
    app_trace_log("Example start\r\n");

    while (true)
    {
        nrf_drv_gpiote_out_set(PIN_OUT_1);
        nrf_delay_ms(1000);
        nrf_drv_gpiote_out_set(PIN_OUT_2);
        nrf_drv_gpiote_out_clear(PIN_OUT_1);
        nrf_delay_ms(1000);
        nrf_drv_gpiote_out_clear(PIN_OUT_2);
        nrf_delay_ms(1000);
    }
}


/** @} */
